/* ==================================================

   Ben Brillat
   Network Programming
   Project 3, TFTPd
   Due: 25 February 2000

   TFTP Server - main.c


   Some code borrowed from Prof. Hollinger's UDP examples
   for Network Programming, Spring 2000, RPI.

   ================================================== */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>


/* ==================================================
   Local Includes
   ================================================== */

#include "network.h"

/* ==================================================
   Function Prototypes
   ================================================== */


/* ==================================================
   Main
   ================================================== */
int main(int argc, char **argv) {

  int tftp_sock;
  struct sockaddr_in tftp_sa;
  int tftp_port;
  int retval;
  int pid;

  if(argc == 2){
    //assume that 1 argument means that there is a port number specified
    tftp_port = atoi(argv[1]);
    //   fprintf(stderr, "port is: %d\n", tftp_port);
    
  }
  else{
    //assume binding to computer-specified port
    tftp_port = 0;
  }

  //bind to the server port
  tftp_sock = do_bind(&tftp_sa, &tftp_port);

  printf("%d\n", ntohs(tftp_sa.sin_port));

  if(tftp_sock < 0){
    //an error happened
    perror("Error binding to well-known port");
    exit(retval);
  }

  while(1) { //begin server loop

    //wait for a "connection" forever
    retval = wait_udp(tftp_sock, -1);
    
    if(retval < 0){
      //an error happened
      perror("Error calling wait_udp");
      exit(retval);
    }
    
    //when we get here, we're ready to receive data from the
    //client that has just sent a request.
    
    pid = fork(); 
    //pid = 0; //DEBUG
    
    if(pid == 0){ //we are the child
      
      //call the server function    
      retval = server(tftp_sock);
      if(retval < 0){
	//something went wrong in the server function
	perror("Error in return from server");
	return -2;
      }
      
      
      return 0; //we don't want the child to go any further.
    }
    else{ //we are the parent
           
      usleep(500); //sleep a half a second - give time for the child
                   //to read the request that just came in


      //clean up children while there are still children to clean up.
      do{
	retval = waitpid(-1, NULL, WNOHANG);
	//note that there may at times be some defunct children lying around,
	//but they will be cleaned up the next time the proxy is used.
      }while(retval > 0); 
   
    }

  } //end server loop

  return 0; //it worked?!  

} //end main()
